from django.core.management.base import BaseCommand
from django.db import transaction
from django.utils.daemonize import become_daemon

import os
import sys
from datetime import datetime
from optparse import make_option

from djapian.models import Change
from djapian import utils
import djapian

@transaction.commit_manually
def update_changes(verbose, timeout, once):
    def after_index(obj):
        if verbose:
            sys.stdout.write('.')
            sys.stdout.flush()

    while True:
        changes = Change.objects.all().order_by("-date")# The objects must be sorted by date
        objs_count = changes.count()

        if objs_count > 0 and verbose:
            print 'There are %d objects to update' % objs_count

        for change in changes:
            indexers = djapian.indexer_map[change.content_type.model_class()]

            for indexer in indexers:
                if change.action == "delete":
                    indexer.delete(change.object_id)
                else:
                    indexer.update([change.object], after_index)
            change.delete()

        # Need to commit if using transactions (e.g. MySQL+InnoDB) since autocommit is
        # turned off by default according to PEP 249. See also:
        # http://dev.mysql.com/doc/refman/5.0/en/innodb-consistent-read-example.html
        #
        # PEP 249 states "Database modules that do not support transactions should
        #                 implement this method with void functionality".
        transaction.commit()

        if once:
            break

        time.sleep(timeout)

def rebuild(verbose):
    def after_index(obj):
        if verbose:
            sys.stdout.write('.')
            sys.stdout.flush()
    for model, indexers in djapian.indexer_map.iteritems():
        for indexer in indexers:
            indexer.update(after_index=after_index)

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--verbose', action='store_true', default=False,
                    help='Verbosity output'),
        make_option("--daemonize", dest="make_daemon", default=False,
                    action="store_true",
                    help="Do not fork the process"),
        make_option("--time-out", dest="timeout", default=10, type="int",
                    help="Time to sleep between each query to the"
                         " database (default: %default)"),
        make_option("--rebuild", dest="rebuild_index", default=False,
                    action="store_true",
                    help="Rebuild index database"),
    )
    help = "This is the Djapian daemon used to update the index based on djapian_change table."

    requires_model_validation = True

    def handle(self, verbose=False, make_daemon=False, timeout=10,
               rebuild_index=False, *args, **options):
        utils.load_indexes()

        if make_daemon:
            become_daemon()

        if rebuild_index:
            rebuild(verbose)
        else:
            update_changes(verbose, timeout, not make_daemon)

        if verbose:
            print '\n'



import os
from datetime import datetime

from django.db import models

from djapian import Indexer, Field
from djapian.tests.utils import BaseTestCase, BaseIndexerTest, Entry, Person

class IndexerUpdateTest(BaseIndexerTest, BaseTestCase):
    def test_database_exists(self):
        self.assert_(os.path.exists(Entry.indexer._db._path))

    def test_document_count(self):
        self.assertEqual(Entry.indexer.document_count(), 1)

class IndexCommandTest(BaseTestCase):
    def setUp(self):
        p = Person.objects.create(name="Alex")
        entry1 = Entry.objects.create(
            author=p,
            title="Test entry",
            text="Not large text field"
        )
        entry2 = Entry.objects.create(
            author=p,
            title="Another test entry",
            is_active=False
        )

        from django.core.management import call_command

        call_command("index", no_fork=True, once=True)

    def test_database(self):
        self.assertEqual(Entry.indexer.document_count(), 1)



import sys
import cmd
from django.core.management.base import BaseCommand

from djapian import utils
import djapian

def with_index(func):
    def _decorator(cmd, arg):
        if cmd._current_index is None:
            print "No index selected"
            return

        return func(cmd, arg)
    _decorator.__doc__ = func.__doc__
    return _decorator

class Interpreter(cmd.Cmd):
    prompt = ">>> "

    def __init__(self, *args):
        self._list = djapian.indexer_map.items()
        self._current_index = None

        if len(args):
            self.do_use(args[0])

        cmd.Cmd.__init__(self)

    def do_list(self, arg):
        """
        Lists all available indexes with their ids
        """
        for i, pair in enumerate(self._list):
            model, indexers = pair
            print "%s: `%s` by:" % (i, utils.model_name(model))
            for j, indexer in enumerate(indexers):
                print "\t%s.%s: %s" % (i, j, indexer)

    def do_exit(self, arg):
        """
        Exit shell
        """
        return True

    def do_use(self, index):
        """
        Changes current index
        """
        model, indexer = map(int, index.split('.'))
        self._current_index = self._list[model][1][indexer]
        print "Using `%s by %s` index." % (utils.model_name(self._list[model][0]), self._list[model][1][indexer])

    @with_index
    def do_query(self, query):
        """
        Returns objects fetched by given query
        """
        print list(self._current_index.search(query))

    @with_index
    def do_count(self, query):
        """
        Returns count of objects fetched by given query
        """
        print self._current_index.search(query).count()

    @with_index
    def do_total(self, arg):
        """
        Returns count of objects in index
        """
        print self._current_index.document_count()

    def do_stats(self, arg):
        """
        Print index status information
        """
        import operator
        print "Number of indexes: %s" % reduce(operator.add, [len(indexes) for model, indexes in self._list])

    @with_index
    def do_docslist(self, slice=""):
        """
        Returns count of objects in index
        """
        db = self._current_index._db.open()

        start, end = self._parse_slice(slice, db.get_lastdocid())

        for i in range(start, end + 1):
            doc = db.get_document(i)
            print "doc #%s:\n\tValues (%s):" % (i, doc.values_count())
            val = doc.values_begin()

            for i in range(doc.values_count()):
                print "\t\t%s: %s" % (val.get_valueno(), val.get_value())
                val.next()

            print "\tTerms (%s):" % doc.termlist_count()
            termlist = doc.termlist_begin()

            for i in range(doc.termlist_count()):
                print termlist.get_term(),
                termlist.next()
            print "\n"

    @with_index
    def do_delete(self, id):
        """
        Removes document by id
        """
        id = int(id)

        db = self._current_index._db.open(write=True)
        db.delete_document(id)
        print "Document #%s deleted." % id

    def _parse_slice(self, slice="", last=None):
        if slice:
            bits = slice.split(':')
            if len(bits) == 2:
                start, end = map(int, bits)
            else:
                start = end = int(bits[0])
        else:
            start = 1
            end = last

        end = min(end, last)

        return start, end

class Command(BaseCommand):
    help = "Djapian shell that provides capabilities to monitoring indexes."
    args = '[index_id]'

    requires_model_validation = True

    def handle(self, *args, **options):
        utils.load_indexes()

        try:
            Interpreter(*args).cmdloop("Interactive Djapian shell.")
        except KeyboardInterrupt:
            print "\n"



import os

from djapian import Field
from djapian.tests.utils import BaseTestCase, BaseIndexerTest, Entry, Person
from djapian.models import Change
from django.utils.encoding import force_unicode

class IndexerTest(BaseTestCase):
    def test_text_fields_count(self):
        self.assertEqual(len(Entry.indexer.fields), 1)

    def test_attributes_count(self):
        self.assertEqual(len(Entry.indexer.tags), 7)

class FieldResolverTest(BaseTestCase):
    def setUp(self):
        p = Person.objects.create(name="Alex")
        self.entry = Entry.objects.create(author=p, title="Test entry")

    def test_simple_attribute(self):
        self.assertEqual(Field("title").resolve(self.entry), "Test entry")

    def test_related_attribute(self):
        self.assertEqual(Field("author.name").resolve(self.entry), "Alex")

    def test_fk_attribute(self):
        self.assertEqual(force_unicode(Field("author").resolve(self.entry)), "Alex")

    def test_method(self):
        self.assertEqual(
            Field("headline").resolve(self.entry),
            "Alex - Test entry"
        )

class ChangeTrackingTest(BaseTestCase):
    def setUp(self):
        p = Person.objects.create(name="Alex")
        Entry.objects.create(author=p, title="Test entry")
        Entry.objects.create(
            author=p,
            title="Another test entry",
            is_active=False
        )

    def test_change_count(self):
        self.assertEqual(Change.objects.count(), 2)

class ChangeTrackingUpdateTest(BaseTestCase):
    def setUp(self):
        p = Person.objects.create(name="Alex")
        entry = Entry.objects.create(author=p, title="Test entry")

        entry.title = "Foobar title"
        entry.save()

    def test_change_count(self):
        self.assertEqual(Change.objects.count(), 1)

    def test_change_action(self):
        self.assertEqual(Change.objects.get().action, "add")

class ChangeTrackingDeleteTest(BaseTestCase):
    def setUp(self):
        p = Person.objects.create(name="Alex")
        entry = Entry.objects.create(author=p, title="Test entry")
        entry.delete()

    def test_change_count(self):
        self.assertEqual(Change.objects.count(), 0)



import xapian

from django.db.models import get_model

from djapian import utils

class defaultdict(dict):
    def __init__(self, value_type):
        self._value_type= value_type

    def __getitem__(self, key):
        if key not in self:
            dict.__setitem__(self, key, list())
        return dict.__getitem__(self, key)

class ResultSet(object):
    def __init__(self, indexer, query_str, offset=0, limit=utils.DEFAULT_MAX_RESULTS,
                 order_by=None, prefetch=False, flags=None):
        self._indexer = indexer
        self._query_str = query_str
        self._offset = offset
        self._limit = limit
        self._order_by = order_by
        self._prefetch = prefetch

        if flags is None:
            flags = xapian.QueryParser.FLAG_PHRASE\
                        | xapian.QueryParser.FLAG_BOOLEAN\
                        | xapian.QueryParser.FLAG_LOVEHATE
        self._flags = flags

        self._resultset_cache = None
        self._mset = None
        self._query = None
        self._query_parser = None

    def spell_correction(self):
        return self._clone(
            flags=self._flags | xapian.QueryParser.FLAG_SPELLING_CORRECTION\
                                | xapian.QueryParser.FLAG_WILDCARD
        )

    def prefetch(self):
        return self._clone(prefetch=True)

    def order_by(self, field):
        return self._clone(order_by=field)

    def flags(self, flags):
        return self._clone(flags=flags)

    #def stemming(self, lang):
    #    return self._clone(stemming_lang=lang)

    def count(self):
        return self._clone()._do_count()

    def get_corrected_query_string(self):
        self._fetch_results()
        return self._query_parser.get_corrected_query_string()

    def _clone(self, **kwargs):
        data = {
            "indexer": self._indexer,
            "query_str": self._query_str,
            "offset": self._offset,
            "limit": self._limit,
            "order_by": self._order_by,
            "prefetch": self._prefetch,
            "flags": self._flags
        }
        keys = data.keys()

        data.update(kwargs)

        return ResultSet(**data)

    def _do_count(self):
        self._fetch_results()

        return self._mset.size()

    def _do_prefetch(self):
        model_map = defaultdict(list)

        for hit in self._resultset_cache:
            model_map[hit.model].append(hit)

        for model, hits in model_map.iteritems():
            pks = [hit.pk for hit in hits]

            instances = model._default_manager.in_bulk(pks)

            for hit in hits:
                hit.instance = instances[hit.pk]

    def _fetch_results(self):
        if self._resultset_cache is None:
            self._mset, self._query, self._query_parser = self._indexer._do_search(
                self._query_str,
                self._offset,
                self._limit,
                self._order_by,
                self._flags
            )
            self._parse_results()

        return self._resultset_cache

    def _parse_results(self):
        self._resultset_cache = []

        for match in self._mset:
            doc = match.get_document()

            model = doc.get_value(2)
            model = get_model(*model.split('.'))
            pk = model._meta.pk.to_python(doc.get_value(1))

            percent = match.get_percent()
            rank = match.get_rank()
            weight = match.get_weight()

            self._resultset_cache.append(Hit(pk, model, percent, rank, weight))

        if self._prefetch:
            self._do_prefetch()

    def __iter__(self):
        self._fetch_results()
        return iter(self._resultset_cache)

    def __len__(self):
        self._fetch_results()
        return len(self._resultset_cache)

    def __getitem__(self, k):
        if not isinstance(k, (slice, int, long)):
            raise TypeError
        assert ((not isinstance(k, slice) and (k >= 0))
                or (isinstance(k, slice) and (k.start is None or k.start >= 0)
                    and (k.stop is None or k.stop >= 0))), \
                "Negative indexing is not supported."

        if self._resultset_cache is not None:
            return self._fetch_results()[k]
        else:
            if isinstance(k, slice):
                start, stop = k.start, k.stop
                if start is None:
                    start = 0
                if stop is None:
                    kstop = utils.DEFAULT_MAX_RESULTS

                return self._clone(
                    offset=start,
                    limit=stop - start
                )
            else:
                return list(self._clone(
                    offset=k,
                    limit=1
                ))[k]

        def __unicode__(self):
            return "<ResultSet: query=%s prefetch=%s>" % (self.query_str, self._prefetch)

class Hit(object):
    def __init__(self, pk, model, percent, rank, weight):
        self.pk = pk
        self.model = model
        self.percent = percent
        self.rank = rank
        self.weight = weight
        self._instance = None

    def get_instance(self):
        if self._instance is None:
            self._instance = self.model._default_manager.get(pk=self.pk)
        return self._instance

    def set_instance(self, instance):
        self._instance = instance

    instance = property(get_instance, set_instance)

    def __repr__(self):
        return "<Hit: model=%s pk=%s, percent=%s rank=%s weight=%s>" % (
            utils.model_name(self.model), self.pk, self.percent, self.rank, self.weight
        )



# -*- coding: utf-8 -*-
from django.contrib import admin
from djapian.models import Change


class ChangeAdmin(admin.ModelAdmin):
    """Set what's displayed in admin"""
    list_display = ('content_type', 'object_id', 'action', "date", )
    list_filter = ('content_type', 'action', )


admin.site.register(Change, ChangeAdmin)



from django.core.paginator import Paginator

from djapian.tests.utils import BaseTestCase, Entry, Person

class ResultSetPaginationTest(BaseTestCase):
    num_entries = 100
    per_page = 10
    num_pages = num_entries / per_page

    def setUp(self):
        p = Person.objects.create(name="Alex")

        for i in range(self.num_entries):
            Entry.objects.create(
                author=p,
                title="Entry with number %s" % i,
                text="foobar " * i
            )

        Entry.indexer.update()

        self.result = Entry.indexer.search("title:number")

    def test_pagintion(self):
        paginator = Paginator(self.result, self.per_page)

        self.assertEqual(paginator.num_pages, self.num_pages)

        page = paginator.page(5)



# Module taken from Turbion blog engine

import os
import imp

class NoModuleError(Exception):
    """
    Custom exception class indicates that given module does not exit at all
    """
    pass

def get_module(base, module_name):
    try:
        base_path = __import__(base, {}, {}, [base.split('.')[-1]]).__path__
    except AttributeError:
        raise NoModuleError("Cannot load base `%s`" % base)

    try:
        imp.find_module(module_name, base_path)
    except ImportError:
        raise NoModuleError("Cannot find module `%s` in base `%s`" % (module_name, base))

    return getattr(__import__(base, {}, {}, [module_name]), module_name)

def get_module_attrs(base, module_name, filter=lambda attr: True):
    mod = get_module(base, module_name)

    try:
        attrs = mod.__all__
    except AttributeError:
        attrs = dir(mod)

    return dict([(name, getattr(mod, name)) for name in attrs if filter(getattr(mod, name))])

def list_sub_modules(base, module_name):
    mod = get_module(base, module_name)

    directory = os.path.dirname(os.path.abspath(mod.__file__))
    names = [name for name in os.listdir(directory) if name.endswith('.py') and name != '__init__.py']
    return [os.path.splitext(name)[0] for name in names]

def get_sub_modules(base, module_name):
    return [get_module("%s.%s" % (base, module_name), sub_mod_name)\
                    for sub_mod_name in list_sub_modules(base, module_name)]



import datetime
import os

from django.db import models
from django.utils.itercompat import is_iterable
from djapian.signals import post_save, pre_delete
from django.conf import settings
from django.utils.encoding import smart_unicode

from djapian.resultset import ResultSet
from djapian import utils

import xapian

class Field(object):
    raw_types = (int, long, float, basestring, bool, models.Model,
                 datetime.time, datetime.date, datetime.datetime)

    def __init__(self, path, weight=utils.DEFAULT_WEIGHT, prefix=""):
        self.path = path
        self.weight = weight
        self.prefix = prefix

    def get_tag(self):
        return self.prefix.upper()

    def resolve(self, value):
        bits = self.path.split(".")

        for bit in bits:
            try:
                value = getattr(value, bit)
            except AttributeError:
                raise

            if callable(value):
                try:
                    value = value()
                except TypeError:
                    raise

        if isinstance(value, self.raw_types):
            return value
        elif is_iterable(value):
            return ", ".join(value)
        elif isinstance(value, models.Manager):
            return ", ".join(value.all())
        return None

class Indexer(object):
    field_class = Field
    free_values_start_number = 11

    fields = []
    tags = []
    aliases = {}
    trigger = lambda indexer, obj: True
    stemming_lang_accessor = None

    def __init__(self, db, model):
        """
        Initialize an Indexer whose index data to `db`.
        `model` is the Model whose instances will be used as documents.
        Note that fields from other models can still be used in the index,
        but this model will be the one returned from search results.
        """
        self._db = db
        self._model = model
        self._model_name = ".".join([model._meta.app_label, model._meta.object_name.lower()])

        self.fields = [] # Simple text fields
        self.tags = [] # Prefixed fields
        self.aliases = {}

        #
        # Parse fields
        # For each field checks if it is a tuple or a list and add it's
        # weight
        #
        for field in self.__class__.fields:
            if isinstance(field, (tuple, list)):
                self.fields.append(Field(field[0], field[1]))
            else:
                self.fields.append(Field(field))

        #
        # Parse prefixed fields
        #
        for field in self.__class__.tags:
            tag, path = field[:2]
            if len(field) == 3:
                weight = field[2]
            else:
                weight = utils.DEFAULT_WEIGHT

            self.tags.append(Field(path, weight, prefix=tag))

        for tag, aliases in self.__class__.aliases.iteritems():
            if self.has_tag(tag):
                if not isinstance(aliases, (list, tuple)):
                    aliases = (aliases,)
                self.aliases[tag] = aliases
            else:
                raise ValueError("Cannot create alias for tag `%s` that doesn't exist" % tag)

        models.signals.post_save.connect(post_save, sender=self._model)
        models.signals.pre_delete.connect(pre_delete, sender=self._model)

    def __unicode__(self):
        return self.__class__.get_descriptor()
    __str__ = __unicode__

    def has_tag(self, name):
        for field in self.tags:
            if field.prefix == name:
                return True

        return False

    def tag_index(self, name):
        for i, field in enumerate(self.tags):
            if field.prefix == name:
                return i

        return None

    @classmethod
    def get_descriptor(cls):
        return ".".join([cls.__module__, cls.__name__]).lower()

    # Public Indexer interface

    def update(self, documents=None, after_index=None):
        """
        Update the database with the documents.
        There are some default value and terms in a document:
         * Values:
           1. Used to store the ID of the document
           2. Store the model of the object (in the string format, like
              "project.app.model")
           3. Store the indexer descriptor (module path)
           4..10. Free

         * Terms
           UID: Used to store the ID of the document, so we can replace
                the document by the ID
        """
        # Open Xapian Database
        database = self._db.open(write=True)

        # If doesnt have any document get all
        if documents is None:
            update_queue = self._model.objects.all()
        else:
            update_queue = documents

        try:
            iterator = update_queue.iterator()
        except AttributeError:
            iterator = iter(update_queue)

        # Get each document received
        for obj in iterator:
            if not self.trigger(obj):
                self.delete(obj.pk, database)
                continue

            doc = xapian.Document()
            #
            # Add default terms and values
            #
            uid = self._create_uid(obj)
            doc.add_term(self._create_uid(obj))
            self._insert_meta_values(doc, obj)

            generator = xapian.TermGenerator()
            generator.set_database(database)
            generator.set_document(doc)
            generator.set_flags(xapian.TermGenerator.FLAG_SPELLING)

            stem_lang = self._get_stem_language(obj)
            if stem_lang:
                generator.set_stemmer(xapian.Stem(stem_lang))

            valueno = self.free_values_start_number

            for field in self.fields + self.tags:
                # Trying to resolve field value or skip it
                try:
                    value = field.resolve(obj)
                except AttributeError:
                    if field.prefix:
                        valueno += 1

                if field.prefix:
                    # If it is a model field make some postprocessing of its value
                    try:
                        content_type = self._model._meta.get_field(field.path.split('.')[0])
                    except models.FieldDoesNotExist:
                        content_type = value

                    index_value = self._get_index_value(value, content_type)
                    if index_value is not None:
                        doc.add_value(valueno, smart_unicode(index_value))
                    valueno += 1
                prefix = smart_unicode(field.get_tag())
                generator.index_text(smart_unicode(value), field.weight, prefix)
                if prefix:  # if prefixed then also index without prefix
                    generator.index_text(smart_unicode(value), field.weight)

            database.replace_document(uid, doc)
            #FIXME: ^ may raise InvalidArgumentError when word in
            #         text larger than 255 simbols
            if after_index:
                after_index(obj)
        database.flush()
        del database

    def search(self, query):
        return ResultSet(self, query)

    def delete(self, obj, database=None):
        """
        Delete a document from index
        """
        try:
            if database is None:
                database = self._db.open(write=True)
            database.delete_document(self._create_uid(obj))
        except (IOError, RuntimeError, xapian.DocNotFoundError), e:
            pass

    def document_count(self):
        database = self._db.open()
        return database.get_doccount()

    __len__ = document_count

    def clear(self):
        self._db.clear()

    # Private Indexer interface

    def _get_meta_values(self, obj):
        if isinstance(obj, models.Model):
            pk = obj.pk
        else:
            pk = obj
        return [pk, self._model_name, self.__class__.get_descriptor()]

    def _insert_meta_values(self, doc, obj, start=1):
        for value in self._get_meta_values(obj):
            doc.add_value(start, smart_unicode(value))
            start += 1
        return start

    def _create_uid(self, obj):
        """
        Generates document UID for given object
        """
        return "UID-" + "-".join(map(smart_unicode, self._get_meta_values(obj)))

    def _do_search(self, query, offset, limit, order_by, flags):
        """
        flags are as defined in the Xapian API :
        http://www.xapian.org/docs/apidoc/html/classXapian_1_1QueryParser.html
        Combine multiple values with bitwise-or (|).
        """
        database = self._db.open()
        enquire = xapian.Enquire(database)

        if order_by in (None, 'RELEVANCE'):
            enquire.set_sort_by_relevance()
        else:
            ascending = True
            if order_by.startswith('-'):
                ascending = False

            if order_by[0] in '+-':
                order_by = order_by[1:]

            try:
                valueno = self.free_values_start_number + self.tag_index(order_by)
            except (ValueError, TypeError):
                raise ValueError("Field %s cannot be used in order_by clause"
                                 " because it doen't exist in index" % order_by)

            enquire.set_sort_by_relevance_then_value(valueno, ascending)

        query, query_parser = self._parse_query(query, database, flags)
        enquire.set_query(
            query
        )

        return enquire.get_mset(offset, limit), query, query_parser

    def _get_stem_language(self, obj=None):
        """
        Returns stemmig language for given object if acceptable or model wise
        """
        language = getattr(settings, "DJAPIAN_STEMMING_LANG", "none") # Use the language defined in DJAPIAN_STEMMING_LANG

        if obj:
            try:
                language = Field(self.stemming_lang_accessor).resolve(obj)
            except AttributeError:
                pass

        return language

    def _get_index_value(self, field_value, content_type):
        """
        Generates index values (for sorting) for given field value and its content type
        """
        value = field_value

        if isinstance(content_type, (models.IntegerField, int, long)):
            #
            # Integer fields are stored with 12 leading zeros
            #
            value = '%012d' % field_value
        elif isinstance(content_type, (models.BooleanField, bool)):
            #
            # Boolean fields are stored as 't' or 'f'
            #
            if field_value:
                value = 't'
            else:
                value = 'f'
        elif isinstance(content_type, (models.DateTimeField, datetime.datetime)):
            #
            # DateTime fields are stored as %Y%m%d%H%M%S (better
            # sorting)
            #
            value = field_value.strftime('%Y%m%d%H%M%S')

        return value

    def _parse_query(self, term, db, flags):
        """
        Parses search queries
        """
        # Instance Xapian Query Parser
        query_parser = xapian.QueryParser()

        for field in self.tags:
            query_parser.add_prefix(field.prefix.lower(), field.get_tag())
            if field.prefix in self.aliases:
                for alias in self.aliases[field.prefix]:
                    query_parser.add_prefix(alias, field.get_tag())

        query_parser.set_database(db)
        query_parser.set_default_op(xapian.Query.OP_AND)

        stemming_lang = self._get_stem_language()
        if stemming_lang:
            query_parser.set_stemmer(xapian.Stem(stemming_lang))
            query_parser.set_stemming_strategy(xapian.QueryParser.STEM_SOME)

        parsed_query = query_parser.parse_query(term, flags)

        return parsed_query, query_parser



from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

from datetime import datetime

from djapian import utils

class ChangeManager(models.Manager):
    def create(self, object, action, **kwargs):
        ct = ContentType.objects.get_for_model(object.__class__)

        try:
            old_change = self.get(
                content_type=ct,
                object_id=object.pk
            )
            if old_change.action=="add":
                if action=="edit":
                    old_change.save()
                    return old_change
                elif action=="delete":
                    old_change.delete()
                    return None
            old_change.delete()
        except self.model.DoesNotExist:
            old_change = self.model(object=object)

        old_change.action = action
        old_change.save()

        return old_change


class Change(models.Model):
    ACTIOINS = (
        ("add", "object added"),
        ("edit", "object edited"),
        ("delete", "object deleted"),
    )

    content_type = models.ForeignKey(ContentType, db_index=True)
    object_id = models.PositiveIntegerField()
    date = models.DateTimeField(default=datetime.now)
    action = models.CharField(max_length=6, choices=ACTIOINS)

    object = generic.GenericForeignKey()

    objects = ChangeManager()

    def __unicode__(self):
        return u'%s#%d To action:`%s`, added on %s' % (
            self.content_type,
            self.object_id,
            self.action,
            self.date
        )

    def save(self):
        self.date = datetime.now()

        super(Change, self).save()

    class Meta:
        unique_together = [("content_type", "object_id")]



from django.test import TestCase
from djapian.tests.utils import BaseTestCase, BaseIndexerTest, Entry, Person

class IndexerSearchTextTest(BaseIndexerTest, BaseTestCase):
    def setUp(self):
        super(IndexerSearchTextTest, self).setUp()
        self.result = Entry.indexer.search("text")

    def test_result_count(self):
        self.assertEqual(len(self.result), 1)

    def test_result_row(self):
        self.assertEqual(self.result[0].instance, self.entries[0])

    def test_result_list(self):
        self.assertEqual([r.instance for r in self.result], self.entries[0:1])

    def test_score(self):
        self.assert_(self.result[0].percent in (99, 100))

class AliasesTest(BaseTestCase):
    num_entries = 10

    def setUp(self):
        p = Person.objects.create(name="Alex")

        for i in range(self.num_entries):
            Entry.objects.create(author=p, title="Entry with number %s" % i, text="foobar " * i)

        Entry.indexer.update()

        self.result = Entry.indexer.search("subject:number")

    def test_result(self):
        self.assertEqual(len(self.result), 10)

class CorrectedQueryStringTest(BaseIndexerTest, BaseTestCase):
    def test_correction(self):
        results = Entry.indexer.search("texte").spell_correction()

        self.assertEqual(results.get_corrected_query_string(), "text")



from django.test import TestCase

from djapian.tests.utils import BaseIndexerTest, Entry

def query_test(query, count):
    class _QueryTest(BaseIndexerTest, TestCase):
        def setUp(self):
            super(_QueryTest, self).setUp()
            self.result = Entry.indexer.search(query)

        def test_result_count(self):
            self.assertEqual(len(self.result), count)
    _QueryTest.__name__ = _QueryTest.__name__ + '_' + query.replace(" ", "_")
    return _QueryTest

IndexerSearchCharFieldTest = query_test("title:test", 1)
IndexerSearchAliasFieldTest = query_test("subject:test", 1)
IndexerSearchBoolFieldTest = query_test("active:True", 1)
IndexerSearchAndQueryTest = query_test("title:test AND title:another", 0)



"""
Here are the post_save and the pre_delete signals
"""
from djapian.models import Change

def post_save(sender, instance, created, *args, **kwargs):
    '''Create the Change object to update the index'''
    Change.objects.create(object=instance, action= created and "add" or "edit")

def pre_delete(sender, instance, *args, **kwargs):
    '''Create the Change object to update the index'''
    Change.objects.create(object=instance, action="delete")



# -*- coding: utf-8 -*-
import os
from datetime import datetime

from django.db import models
from django.test import TestCase

import djapian
from djapian import Indexer, Field

class Person(models.Model):
    name = models.CharField(max_length=150)

    def __unicode__(self):
        return self.name

    class Meta:
        app_label = "djapian"

class Entry(models.Model):
    author = models.ForeignKey(Person, related_name="entries")
    title = models.CharField(max_length=250)
    tags = models.CharField(max_length=250, null=True)
    created_on = models.DateTimeField(default=datetime.now)

    asset_count = models.PositiveIntegerField(default=0)
    is_active = models.BooleanField(default=True)

    text = models.TextField()

    editors = models.ManyToManyField(Person, related_name="edited_entries")

    def headline(self):
        return "%s - %s" % (self.author, self.title)

    def __unicode__(self):
        return self.title

    class Meta:
        app_label = "djapian"

class EntryIndexer(Indexer):
    fields=["text"]
    tags=[
        ("author", "author.name"),
        ("title", "title", 3),
        ("tag", "tags", 2),
        ("date", "created_on"),
        ("active", "is_active"),
        ("count", "asset_count"),
        ("editors", "editors"),
    ]
    aliases={
        "title": "subject",
        "author": "user",
    }
    trigger=lambda indexer, obj: obj.is_active

indexer = djapian.add_index(Entry, EntryIndexer, attach_as="indexer")

class BaseTestCase(TestCase):
    def tearDown(self):
        Entry.indexer.clear()

class BaseIndexerTest(object):
    def setUp(self):
        p = Person.objects.create(name="Alex")

        self.entries= [
            Entry.objects.create(
                author=p,
                title="Test entry",
                text="Not large text field"
            ),
            Entry.objects.create(
                author=p,
                title="Another test entry",
                is_active=False
            )
        ]

        Entry.indexer.update()



from django.conf import settings

DEFAULT_MAX_RESULTS = 100000
DEFAULT_WEIGHT = 1

def model_name(model):
    return "%s.%s" % (model._meta.app_label, model._meta.object_name)

def load_indexes():
    from djapian.utils import loading
    for app in settings.INSTALLED_APPS:
        try:
            loading.get_module(app, "index")
        except loading.NoModuleError:
            pass



import os

from django.conf import settings

from djapian.indexer import Field, Indexer
from djapian.database import Database
from djapian.utils import load_indexes

indexer_map = {}

def add_index(model, indexer=None, attach_as=None, db=None):
    if db is None:
        db = Database(
            os.path.join(
                settings.DJAPIAN_DATABASE_PATH,
                model._meta.app_label,
                model._meta.object_name.lower(),
                indexer.get_descriptor()
            )
        )

    indexer = db.add_index(model, indexer, attach_as)

    if model in indexer_map:
        indexer_map[model].append(indexer)
    else:
        indexer_map[model] = [indexer]

    return indexer



#!/usr/bin/env python

from distutils.core import setup

setup(
    name="Djapian",
    version="2.0",

    license="New BSD License",

    author='Alex Koshelev',
    author_email="daevaorn@gmail.com",

    #maintainer='Rafael "SDM" Sierra',
    #maintainer_email="rafaeljsg14@gmail.com",

    url="http://code.google.com/p/djapian/",

    packages=[
        "djapian",
        "djapian.utils",
        "djapian.tests",
        "djapian.management",
        "djapian.management.commands"
    ],
    package_dir={
        "djapian": "src/djapian"
    },

    description="High-level Xapian full-text indexer integration for Django",

    classifiers=[
        "Framework :: Django",
        "License :: OSI Approved :: BSD License",
        "Topic :: Text Processing :: Indexing"
    ]
)



import os
import xapian

from django.conf import settings

class Database(object):
    def __init__(self, path):
        self._path = path
        self._indexes = {}

    def add_index(self, model, indexer=None, attach_as="indexer"):
        if indexer is None:
            indexer = self.create_default_indexer(model)

        indexer = indexer(self, model)

        self._indexes[model] = indexer

        if attach_as is not None:
            if hasattr(model, attach_as):
                raise ValueError("Attribute with name `%s` is already exsits" % attach_as)
            else:
                model.add_to_class(attach_as, indexer)
        return indexer

    def open(self, write=False):
        """
        Opens database for manipulations
        """
        if not os.path.exists(self._path):
            os.makedirs(self._path)

        if write:
            database = xapian.WritableDatabase(
                self._path,
                xapian.DB_CREATE_OR_OPEN,
            )
        else:
            try:
                database = xapian.Database(self._path)
            except xapian.DatabaseOpeningError:
                self._create_database()

                database = xapian.Database(self._path)

        return database

    def _create_database(self):
        database = xapian.WritableDatabase(
            self._path,
            xapian.DB_CREATE_OR_OPEN,
        )
        del database

    def document_count(self):
        pass

    def create_default_indexer(self, model):
        pass

    def index_count(self):
        return len(self._indexes)

    def clear(self):
        try:
            for file_path in os.listdir(self._path):
                os.remove(os.path.join(self._path, file_path))

            os.rmdir(self._path)
        except OSError:
            pass
